(ns org.opencase.clojure.emfquery)

(defn- feature-from-key [eclass key]
  (.getEStructuralFeature eclass (name key)))

(defn- value-seq [object key]
  (let [feature (feature-from-key (.eClass object) key)
        value   (.eGet object feature)]
    (if (.isMany feature)
      (seq value)
      (list value))))

(defn- join [col x]
  (if (seq? x)
    (concat col x)
    (concat col (list x))))


(defn mselect [object param & params]
  (let [key    (if (keyword? param) 
                 param
                 (first param))
        value  (value-seq object key)
        trans  (if (keyword? param) 
                 value
                 ((second param) value))]
    (if (nil? params)
      trans
      (mapcat #(apply mselect (list* % params)) trans))))

(defn value [object key]
  (let [feature (feature-from-key (.eClass object) key)]
    (.eGet object feature)))

(defn eclass [object]
  (keyword (.getName (.eClass object))))

(defn values-of-class [object key cls]
  (filter #(= (eclass %) cls) (value-seq object key)))

(defn enum-to-keyword [value]
  (keyword (.toString value)))

(defn parent [object]
   (.eContainer object))
