(ns org.opencase.reporting.generator
  (:use org.opencase.reporting.report
        org.opencase.reporting.extensions)
  (:import [org.eclipse.emf.common.util URI]
           [org.eclipse.emf.ecore.util EcoreUtil]
           [org.opencase.reporting.report Report Paragraph]))

(defn func-from-element [element] 
  (let [ns   (.getAttribute element "namespace")
        func (.getAttribute element "function")]
    (require (symbol ns))
    (ns-resolve *ns* (symbol (str ns "/" func)))))

(defn uri-from-element [element] 
  (URI/createURI (.getAttribute element "diagramEClass")))
        
(defn create-reporters-map []
  (let [elements (configuration-elements-for "org.opencase.reporting.diagram_reporters")
        uris  (map uri-from-element elements)
        funcs (map func-from-element elements)]
    (zipmap uris funcs)))

(defn report-diagram [diagram reporters-map]
  (let [class-uri (EcoreUtil/getURI (.eClass diagram))
        reporter  (reporters-map class-uri)]
    (reporter diagram nil)))

(defn generate [diagrams file params]           
  (let [reporters-map (create-reporters-map)
        reports       (map #(report-diagram % reporters-map) (seq diagrams))]  
    (Report. "Operational report" :operation [] reports)))



  