(ns org.opencase.ordiagram.reporting.convertors.html
  (:use clojure.data.xml
        org.opencase.reporting.convertors.html))

(defmethod convert org.opencase.ordiagram.reporting.report.ServiceActivity [activity] 
  (element-list :div 
                {:class "service-activity"}  
                (if (empty? (:conditions activity))
                  []
                  [(element-list :div 
                                 {:class "conditions"} 
                                 (map #(element :div {:class "condition"} %) (:conditions activity)))])
                [(element :span {:class "title"} (:title activity))]
                (map #(element :p {:class "communication"} %) (:communications activity))))

(defmulti convert-cell (fn [cell size] (type cell)))

(defmethod convert-cell org.opencase.ordiagram.reporting.report.Activity [activity size]   
  (element-list :td 
                {:class (str "cell" size " activity")}  
                (if (empty? (:conditions activity))
                  []
                  [(element-list :div 
                                 {:class "conditions"} 
                                 (map #(element :div {:class "condition"} %) (:conditions activity)))])
                [(element :span {:class "pos"} (str (:position activity) \)))
                 (element :span {:class "title"} (:title activity))]
                (map #(element :p {:class "communication"} %) (:communications activity))
                [(if-not (nil? (:goto activity))
                   (element :div {:class "goto"} (:goto activity))
                   nil)])) 

(defmethod convert-cell org.opencase.ordiagram.reporting.report.State [state size] 
  (element :td {:class (str "cell" size " state")} 
    (element :span {:class "pos"} (str (:position state) \)))
    (element :span {:class "title"} (:title state))
    (if-not (nil? (:goto state))
      (element :div {:class "goto"} (:goto state))
      nil)))

(defn convert-cells [cells]
  (let [size (count cells)]
    (apply element (list* :tr {} (map convert-cell cells (repeat size))))))
  
(defmethod convert-cell org.opencase.ordiagram.reporting.report.Slot [slot size]
  (if (empty? (:cells slot))
    (element :td {:class "null"})
    (element :td {:class (str "cell" size)}
      (element :table {:class "slot" :cellspacing "0" :cellpadding "0"} 
        (convert-cells (:cells slot))))))

(defmethod convert org.opencase.ordiagram.reporting.report.Level [level]
  (element :table {:class "level" :cellspacing "0" :cellpadding "0"} 
    (element :tr {} 
      (element :td {:class "side"} (str "§" (:index level)))
      (convert-cell (:cell level) 1))))

(defmethod convert org.opencase.ordiagram.reporting.report.StatefulParticipant [part]
  (element :div {:class "participant"} 
    (element :div {:class "part-preamble"} 
      (element :div {:class "title"} (:title part))
      (element :div {:class "info"} (:info part))) 
    (convert-collection :div {:class "levels"} (:levels part))))

(defmethod convert org.opencase.ordiagram.reporting.report.ServiceParticipant [part]
  (element :div {:class "participant"} 
    (element :div {:class "part-preamble"} 
      (element :div {:class "title"} (:title part))
      (element :div {:class "info"} (:info part)))
    (convert-collection :div {:class "service-activities"} (:activities part)))) 

(defmethod convert org.opencase.ordiagram.reporting.report.ORDiagram [diagram]
  (element :div {:class "or-diagram"} 
    (element :div {:class "or-preamble"} 
       (element :div {:class "title"} (:title diagram))) 
    (convert-collection :div {:class "participants"} (:participants diagram))))

(defn or-convert [diagram]
  (convert diagram))

(defn load-css []
  (load-resource "xhtml/ordiagram.css"))
  


